var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
import fs from 'fs';
import path from 'path';
import { createRelayNode } from '../relay.js';
import { RELAY_DEFAULT_HOST, RELAY_DEFAULT_PORT, RELAY_DEFAULT_MAX_DIAL_RETRY, RELAY_REDIAL_INTERVAL, DEFAULT_PING_INTERVAL, DIAL_TIMEOUT } from '../constants.js';
function main() {
    return __awaiter(this, void 0, void 0, function () {
        var argv, peerIdObj, relayPeersList, denyMultiaddrsList, peerIdFilePath, peerIdJson, relayPeersFilePath, relayPeersListObj, denyMultiaddrsFilePath, denyMultiaddrsListObj, relayNodeInit;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    argv = _getArgv();
                    relayPeersList = [];
                    denyMultiaddrsList = [];
                    if (argv.peerIdFile) {
                        peerIdFilePath = path.resolve(argv.peerIdFile);
                        console.log("Reading peer id from file ".concat(peerIdFilePath));
                        peerIdJson = fs.readFileSync(peerIdFilePath, 'utf-8');
                        peerIdObj = JSON.parse(peerIdJson);
                    }
                    else {
                        console.log('Creating a new peer id');
                    }
                    if (argv.relayPeers) {
                        relayPeersFilePath = path.resolve(argv.relayPeers);
                        if (!fs.existsSync(relayPeersFilePath)) {
                            console.log("File at given path ".concat(relayPeersFilePath, " not found, exiting"));
                            process.exit();
                        }
                        console.log("Reading relay peer multiaddr(s) from file ".concat(relayPeersFilePath));
                        relayPeersListObj = fs.readFileSync(relayPeersFilePath, 'utf-8');
                        relayPeersList = JSON.parse(relayPeersListObj);
                    }
                    if (argv.denyMultiaddrs) {
                        denyMultiaddrsFilePath = path.resolve(argv.denyMultiaddrs);
                        if (!fs.existsSync(denyMultiaddrsFilePath)) {
                            console.log("File at given path ".concat(denyMultiaddrsFilePath, " not found, exiting"));
                            process.exit();
                        }
                        console.log("Reading blacklisted multiaddr(s) from file ".concat(denyMultiaddrsFilePath));
                        denyMultiaddrsListObj = fs.readFileSync(denyMultiaddrsFilePath, 'utf-8');
                        denyMultiaddrsList = JSON.parse(denyMultiaddrsListObj);
                    }
                    relayNodeInit = {
                        host: argv.host,
                        port: argv.port,
                        peerIdObj: peerIdObj,
                        announceDomain: argv.announce,
                        relayPeers: relayPeersList,
                        denyMultiaddrs: denyMultiaddrsList,
                        dialTimeout: argv.dialTimeout,
                        pingInterval: argv.pingInterval,
                        redialInterval: argv.redialInterval,
                        maxDialRetry: argv.maxDialRetry,
                        pubsub: argv.pubsub,
                        enableDebugInfo: argv.enableDebugInfo
                    };
                    return [4 /*yield*/, createRelayNode(relayNodeInit)];
                case 1:
                    _a.sent();
                    return [2 /*return*/];
            }
        });
    });
}
function _getArgv() {
    return yargs(hideBin(process.argv)).parserConfiguration({
        'parse-numbers': false
    }).options({
        host: {
            type: 'string',
            alias: 'h',
            default: RELAY_DEFAULT_HOST,
            describe: 'Host to bind to'
        },
        port: {
            type: 'number',
            alias: 'p',
            default: RELAY_DEFAULT_PORT,
            describe: 'Port to start listening on'
        },
        announce: {
            type: 'string',
            alias: 'a',
            describe: 'Domain name to be used in the announce address'
        },
        peerIdFile: {
            type: 'string',
            alias: 'f',
            describe: 'Relay Peer Id file path (json)'
        },
        relayPeers: {
            type: 'string',
            alias: 'r',
            describe: 'Relay peer multiaddr(s) list file path (json)'
        },
        denyMultiaddrs: {
            type: 'string',
            describe: 'Blacklisted multiaddr(s) list file path (json)'
        },
        pingInterval: {
            type: 'number',
            describe: 'Interval to check relay peer connections using ping (ms)',
            default: DEFAULT_PING_INTERVAL
        },
        dialTimeout: {
            type: 'number',
            describe: 'Timeout for dial to relay peers (ms)',
            default: DIAL_TIMEOUT
        },
        redialInterval: {
            type: 'number',
            describe: 'Redial interval to relay peer on connection failure (ms)',
            default: RELAY_REDIAL_INTERVAL
        },
        maxDialRetry: {
            type: 'number',
            describe: 'Maximum number of dial retries to be attempted to a relay peer',
            default: RELAY_DEFAULT_MAX_DIAL_RETRY
        },
        pubsub: {
            type: 'string',
            describe: "Pubsub to use ('floodsub' | 'gossipsub')"
        },
        enableDebugInfo: {
            type: 'boolean',
            describe: "Whether to broadcast node's info over floodsub on request"
        }
        // https://github.com/yargs/yargs/blob/main/docs/typescript.md?plain=1#L83
    }).parseSync();
}
main().catch(function (err) {
    console.log(err);
});
//# sourceMappingURL=relay.js.map