/// <reference types="node" resolution-mode="require"/>
import { Libp2p } from '@cerc-io/libp2p';
import type { PeerId } from '@libp2p/interface-peer-id';
export declare const MUXER_CLOSED_ERR = "Muxer already closed";
interface PeerData {
    intervalId: NodeJS.Timer;
    latencyValues: Array<number>;
}
interface PeerHearbeatCheckerOptions {
    pingInterval: number;
    pingTimeout: number;
}
/**
 * Used for tracking heartbeat of connected remote peers
 */
export declare class PeerHearbeatChecker {
    _node: Libp2p;
    _pingInterval: number;
    _pingTimeout: number;
    _peerMap: Map<string, PeerData>;
    constructor(node: Libp2p, options?: Partial<PeerHearbeatCheckerOptions>);
    /**
     * Method to start heartbeat checks for a peer
     * @param peerId
     * @param handleDisconnect
     */
    start(peerId: PeerId, handleDisconnect: () => Promise<void>): Promise<void>;
    /**
     * Method to stop heartbeat checks for a peer
     * @param peerId
     */
    stop(peerId: PeerId): void;
    /**
     * Get latency data for peer
     */
    getLatencyData(peerId: PeerId): Array<number>;
    /**
     * Method to check connection using ping request
     * @param peerId
     * @param handleDisconnect
     */
    _validatePing(peerId: PeerId, handleDisconnect: () => Promise<void>): Promise<void>;
}
export {};
