//
// Copyright 2023 Vulcanize, Inc.
//
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import debug from 'debug';
import { DEFAULT_PING_INTERVAL, DEFAULT_PING_TIMEOUT } from './constants.js';
import { getPseudonymForPeerId } from './utils/index.js';
var log = debug('laconic:peer-heartbeat-checker');
export var MUXER_CLOSED_ERR = 'Muxer already closed';
/**
 * Used for tracking heartbeat of connected remote peers
 */
var PeerHearbeatChecker = /** @class */ (function () {
    function PeerHearbeatChecker(node, options) {
        if (options === void 0) { options = {}; }
        var _a, _b;
        this._peerMap = new Map();
        this._node = node;
        this._pingInterval = (_a = options.pingInterval) !== null && _a !== void 0 ? _a : DEFAULT_PING_INTERVAL;
        this._pingTimeout = (_b = options.pingTimeout) !== null && _b !== void 0 ? _b : DEFAULT_PING_TIMEOUT;
    }
    /**
     * Method to start heartbeat checks for a peer
     * @param peerId
     * @param handleDisconnect
     */
    PeerHearbeatChecker.prototype.start = function (peerId, handleDisconnect) {
        return __awaiter(this, void 0, void 0, function () {
            var peerIdString, handlePingDisconnect, intervalId;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        peerIdString = peerId.toString();
                        if (this._peerMap.has(peerIdString)) {
                            // Do not start connection check interval if already present
                            return [2 /*return*/];
                        }
                        handlePingDisconnect = function () { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        // Check if connection check interval for peer is already cleared
                                        if (!this._peerMap.get(peerIdString)) {
                                            return [2 /*return*/];
                                        }
                                        // Clear and remove check interval for remote peer if not connected
                                        this.stop(peerId);
                                        return [4 /*yield*/, handleDisconnect()];
                                    case 1:
                                        _a.sent();
                                        return [2 /*return*/];
                                }
                            });
                        }); };
                        intervalId = setInterval(function () { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, this._validatePing(peerId, handlePingDisconnect)];
                                    case 1:
                                        _a.sent();
                                        return [2 /*return*/];
                                }
                            });
                        }); }, this._pingInterval);
                        this._peerMap.set(peerIdString, {
                            intervalId: intervalId,
                            latencyValues: []
                        });
                        return [4 /*yield*/, this._validatePing(peerId, handlePingDisconnect)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Method to stop heartbeat checks for a peer
     * @param peerId
     */
    PeerHearbeatChecker.prototype.stop = function (peerId) {
        // Clear check interval for disconnected peer
        var peerData = this._peerMap.get(peerId.toString());
        if (peerData) {
            clearInterval(peerData.intervalId);
        }
        this._peerMap.delete(peerId.toString());
    };
    /**
     * Get latency data for peer
     */
    PeerHearbeatChecker.prototype.getLatencyData = function (peerId) {
        var _a;
        var latencyValues = (_a = this._peerMap.get(peerId.toString())) === null || _a === void 0 ? void 0 : _a.latencyValues;
        return latencyValues !== null && latencyValues !== void 0 ? latencyValues : [];
    };
    /**
     * Method to check connection using ping request
     * @param peerId
     * @param handleDisconnect
     */
    PeerHearbeatChecker.prototype._validatePing = function (peerId, handleDisconnect) {
        var _a;
        return __awaiter(this, void 0, void 0, function () {
            var pingRetriesOnFail, pingSuccess, i, retryDelayPromise, latency, latencyValues, length, err_1, conns;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        pingRetriesOnFail = Math.floor(this._pingInterval / this._pingTimeout);
                        pingSuccess = false;
                        i = 0;
                        _b.label = 1;
                    case 1:
                        if (!(!pingSuccess && (i < pingRetriesOnFail))) return [3 /*break*/, 10];
                        retryDelayPromise = new Promise(function (resolve) { return setTimeout(resolve, _this._pingTimeout); });
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 4, , 9]);
                        return [4 /*yield*/, this._node.ping(peerId)];
                    case 3:
                        latency = _b.sent();
                        pingSuccess = true;
                        latencyValues = (_a = this._peerMap.get(peerId.toString())) === null || _a === void 0 ? void 0 : _a.latencyValues;
                        if (latencyValues) {
                            length = latencyValues.unshift(latency);
                            if (length > 5) {
                                // Pop oldest latency value from list
                                latencyValues.pop();
                            }
                        }
                        return [3 /*break*/, 9];
                    case 4:
                        err_1 = _b.sent();
                        // On error i.e. no pong
                        log(err_1 === null || err_1 === void 0 ? void 0 : err_1.message);
                        conns = this._node.getConnections(peerId);
                        if (!(conns.length > 1 && (err_1 === null || err_1 === void 0 ? void 0 : err_1.message.includes(MUXER_CLOSED_ERR)))) return [3 /*break*/, 6];
                        log("Closing a dead connection ".concat(conns[0].id, " to peer ").concat(peerId.toString()));
                        return [4 /*yield*/, conns[0].close()];
                    case 5:
                        _b.sent();
                        // Don't count this attempt and do the next ping attempt immediately
                        i--;
                        return [3 /*break*/, 8];
                    case 6: 
                    // Retry after a delay of pingTimeout in case ping fails
                    return [4 /*yield*/, retryDelayPromise];
                    case 7:
                        // Retry after a delay of pingTimeout in case ping fails
                        _b.sent();
                        _b.label = 8;
                    case 8: return [3 /*break*/, 9];
                    case 9:
                        i++;
                        return [3 /*break*/, 1];
                    case 10:
                        if (pingSuccess) {
                            return [2 /*return*/];
                        }
                        log("Not connected to peer ".concat(peerId.toString(), " (").concat(getPseudonymForPeerId(peerId.toString()), ")"));
                        return [4 /*yield*/, handleDisconnect()];
                    case 11:
                        _b.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return PeerHearbeatChecker;
}());
export { PeerHearbeatChecker };
//# sourceMappingURL=peer-heartbeat-checker.js.map