//
// Copyright 2022 Vulcanize, Inc.
//
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import debug from 'debug';
import assert from 'assert';
import { toString as uint8ArrayToString } from 'uint8arrays/to-string';
import { createLibp2p } from '@cerc-io/libp2p';
import { noise } from '@chainsafe/libp2p-noise';
import { mplex } from '@libp2p/mplex';
import { webSockets } from '@libp2p/websockets';
import { pubsubPeerDiscovery } from '@libp2p/pubsub-peer-discovery';
import { multiaddr } from '@multiformats/multiaddr';
import { createFromJSON } from '@libp2p/peer-id-factory';
import { PrometheusMetrics } from '@cerc-io/prometheus-metrics';
import { HOP_TIMEOUT, DEFAULT_PING_TIMEOUT, PUBSUB_DISCOVERY_INTERVAL, MAX_CONCURRENT_DIALS_PER_PEER, DEBUG_INFO_TOPIC } from './constants.js';
import { PeerHearbeatChecker } from './peer-heartbeat-checker.js';
import { debugInfoRequestHandler, dialWithRetry, getConnectionsInfo, getPseudonymForPeerId, getSelfInfo, initPubsub, isMultiaddrBlacklisted } from './utils/index.js';
var log = debug('laconic:relay');
export function createRelayNode(init) {
    var _a;
    return __awaiter(this, void 0, void 0, function () {
        var listenMultiaddrs, announceMultiaddrs, peerId, pingTimeout, metrics, node, peerHeartbeatChecker;
        var _this = this;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    listenMultiaddrs = ["/ip4/".concat(init.host, "/tcp/").concat(init.port, "/ws")];
                    announceMultiaddrs = [];
                    if (init.announceDomain) {
                        announceMultiaddrs.push("/dns4/".concat(init.announceDomain, "/tcp/443/wss"));
                    }
                    if (!init.peerIdObj) return [3 /*break*/, 2];
                    return [4 /*yield*/, createFromJSON(init.peerIdObj)];
                case 1:
                    peerId = _b.sent();
                    _b.label = 2;
                case 2:
                    pingTimeout = (_a = init.pingTimeout) !== null && _a !== void 0 ? _a : DEFAULT_PING_TIMEOUT;
                    metrics = new PrometheusMetrics();
                    return [4 /*yield*/, createLibp2p({
                            peerId: peerId,
                            addresses: {
                                listen: listenMultiaddrs,
                                announce: announceMultiaddrs
                            },
                            transports: [webSockets()],
                            connectionEncryption: [noise()],
                            streamMuxers: [mplex()],
                            pubsub: initPubsub(init.pubsub),
                            peerDiscovery: [
                                pubsubPeerDiscovery({
                                    interval: PUBSUB_DISCOVERY_INTERVAL
                                })
                            ],
                            relay: {
                                enabled: true,
                                hop: {
                                    enabled: true,
                                    timeout: HOP_TIMEOUT
                                }
                            },
                            webRTCSignal: {
                                enabled: true,
                                isSignallingNode: true
                            },
                            connectionManager: {
                                maxDialsPerPeer: MAX_CONCURRENT_DIALS_PER_PEER,
                                autoDial: false,
                                dialTimeout: init.dialTimeout,
                                deny: init.denyMultiaddrs
                            },
                            ping: {
                                timeout: pingTimeout
                            },
                            metrics: function () { return metrics; }
                        })];
                case 3:
                    node = _b.sent();
                    peerHeartbeatChecker = new PeerHearbeatChecker(node, {
                        pingInterval: init.pingInterval,
                        pingTimeout: pingTimeout
                    });
                    log("Relay node started with id ".concat(node.peerId.toString(), " (").concat(getPseudonymForPeerId(node.peerId.toString()), ")"));
                    log('Listening on:');
                    node.getMultiaddrs().forEach(function (ma) { return log(ma.toString()); });
                    // Listen for peers connection
                    node.addEventListener('peer:connect', function (evt) { return __awaiter(_this, void 0, void 0, function () {
                        var connection;
                        var _this = this;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    connection = evt.detail;
                                    log("Connected to ".concat(connection.remotePeer.toString(), " (").concat(getPseudonymForPeerId(connection.remotePeer.toString()), ") using multiaddr ").concat(connection.remoteAddr.toString()));
                                    // Start heartbeat check for peer
                                    return [4 /*yield*/, peerHeartbeatChecker.start(connection.remotePeer, function () { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                                            return [2 /*return*/, _handleDeadConnections(node, connection.remotePeer)];
                                        }); }); })];
                                case 1:
                                    // Start heartbeat check for peer
                                    _a.sent();
                                    return [2 /*return*/];
                            }
                        });
                    }); });
                    // Listen for peers disconnecting
                    // peer:disconnect event is trigerred when all connections to a peer close
                    // https://github.com/libp2p/js-libp2p-interfaces/blob/master/packages/interface-libp2p/src/index.ts#L64
                    node.addEventListener('peer:disconnect', function (evt) { return __awaiter(_this, void 0, void 0, function () {
                        var connection, remoteAddr;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    connection = evt.detail;
                                    remoteAddr = connection.remoteAddr;
                                    log("Disconnected from ".concat(connection.remotePeer.toString(), " (").concat(getPseudonymForPeerId(connection.remotePeer.toString()), ") using multiaddr ").concat(remoteAddr.toString()));
                                    // Stop connection check for disconnected peer
                                    peerHeartbeatChecker.stop(connection.remotePeer);
                                    if (!init.relayPeers.includes(remoteAddr.toString())) return [3 /*break*/, 2];
                                    return [4 /*yield*/, dialWithRetry(node, remoteAddr, {
                                            redialInterval: init.redialInterval,
                                            maxRetry: init.maxDialRetry
                                        }).catch(function (error) { return log(error.message); })];
                                case 1:
                                    _a.sent();
                                    _a.label = 2;
                                case 2: return [2 /*return*/];
                            }
                        });
                    }); });
                    if (!init.relayPeers.length) return [3 /*break*/, 5];
                    log('Dialling relay peers');
                    return [4 /*yield*/, _dialRelayPeers(node, init.relayPeers, init.denyMultiaddrs, init.maxDialRetry, init.redialInterval)];
                case 4:
                    _b.sent();
                    _b.label = 5;
                case 5:
                    if (!init.enableDebugInfo) return [3 /*break*/, 7];
                    log('Debug info enabled');
                    return [4 /*yield*/, _subscribeToDebugTopic(node, peerHeartbeatChecker, metrics)];
                case 6:
                    _b.sent();
                    _b.label = 7;
                case 7: return [2 /*return*/, node];
            }
        });
    });
}
function _dialRelayPeers(node, relayPeersList, denyMultiaddrs, maxDialRetry, redialInterval) {
    return __awaiter(this, void 0, void 0, function () {
        var _this = this;
        return __generator(this, function (_a) {
            relayPeersList.forEach(function (relayPeer) { return __awaiter(_this, void 0, void 0, function () {
                var relayMultiaddr;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            relayMultiaddr = multiaddr(relayPeer);
                            if (isMultiaddrBlacklisted(denyMultiaddrs, relayMultiaddr)) {
                                log("Ignoring blacklisted node with multiaddr ".concat(relayMultiaddr.toString()));
                                return [2 /*return*/];
                            }
                            return [4 /*yield*/, dialWithRetry(node, relayMultiaddr, {
                                    redialInterval: redialInterval,
                                    maxRetry: maxDialRetry
                                }).catch(function (error) { return log(error.message); })];
                        case 1:
                            _a.sent();
                            return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    });
}
function _handleDeadConnections(node, remotePeerId) {
    return __awaiter(this, void 0, void 0, function () {
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    // Close existing connections of remote peer
                    log("Closing connections for ".concat(remotePeerId, " (").concat(getPseudonymForPeerId(remotePeerId.toString()), ")"));
                    return [4 /*yield*/, node.hangUp(remotePeerId)];
                case 1:
                    _a.sent();
                    log('Closed');
                    return [2 /*return*/];
            }
        });
    });
}
function _subscribeToDebugTopic(node, peerHeartbeatChecker, metrics) {
    return __awaiter(this, void 0, void 0, function () {
        var _this = this;
        return __generator(this, function (_a) {
            node.pubsub.subscribe(DEBUG_INFO_TOPIC);
            // Listen for pubsub messages
            node.pubsub.addEventListener('message', function (evt) { return __awaiter(_this, void 0, void 0, function () {
                var msg, dataObj;
                var _this = this;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            msg = evt.detail;
                            // Messages should be signed since globalSignaturePolicy is set to 'StrictSign'
                            assert(msg.type === 'signed');
                            if (!(msg.topic === DEBUG_INFO_TOPIC)) return [3 /*break*/, 2];
                            dataObj = JSON.parse(uint8ArrayToString(msg.data));
                            return [4 /*yield*/, debugInfoRequestHandler({
                                    node: node,
                                    getPeerInfo: function () { return __awaiter(_this, void 0, void 0, function () { return __generator(this, function (_a) {
                                        return [2 /*return*/, _getRelayPeerInfo(node, peerHeartbeatChecker, metrics)];
                                    }); }); },
                                    peerId: msg.from,
                                    msg: dataObj
                                })];
                        case 1:
                            _a.sent();
                            _a.label = 2;
                        case 2: return [2 /*return*/];
                    }
                });
            }); });
            return [2 /*return*/];
        });
    });
}
function _getRelayPeerInfo(node, peerHeartbeatChecker, metrics) {
    return __awaiter(this, void 0, void 0, function () {
        var selfInfo, connInfo, metricsMap;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    selfInfo = getSelfInfo(node);
                    connInfo = getConnectionsInfo(node, peerHeartbeatChecker);
                    return [4 /*yield*/, metrics.getMetricsAsMap()];
                case 1:
                    metricsMap = _a.sent();
                    return [2 /*return*/, {
                            selfInfo: selfInfo,
                            connInfo: connInfo,
                            metrics: metricsMap
                        }];
            }
        });
    });
}
//# sourceMappingURL=relay.js.map