import { Libp2p } from '@cerc-io/libp2p';
import { Multiaddr } from '@multiformats/multiaddr';
import type { PeerId } from '@libp2p/interface-peer-id';
import type { Connection } from '@libp2p/interface-connection';
import { ConnectionInfo, DebugPeerInfo, SelfInfo } from '../types/debug-info.js';
import { PeerHearbeatChecker } from '../peer-heartbeat-checker.js';
export declare const CODE_P2P = 421;
export declare const CODE_CIRCUIT = 290;
export type PubsubType = 'floodsub' | 'gossipsub';
interface DialWithRetryOptions {
    redialInterval: number;
    maxRetry: number;
}
/**
 * Method to dial remote peer multiaddr with retry on failure
 * Number of retries can be configured using options.maxRetry
 * @param node
 * @param multiaddr
 * @param options
 */
export declare const dialWithRetry: (node: Libp2p, multiaddr: Multiaddr, options: Partial<DialWithRetryOptions>) => Promise<Connection>;
/**
 * Get a deterministic pseudonym of form [adjective-color-name] for a given libp2p peer id
 * Eg. 12D3KooWJLXEX2GfHPSZR3z9QKNSN8EY6pXo7FZ9XtFhiKLJATtC -> jolly-green-diann
 * @param peerId
 */
export declare const getPseudonymForPeerId: (peerId: string) => string;
/**
 * Method to check if the given multiaddr is part of a blacklist
 * @param blacklist
 * @param multiaddr
 */
export declare const isMultiaddrBlacklisted: (blacklist: string[], multiaddr: Multiaddr) => boolean;
/**
 * Handler for pubsub debug info request
 * @param peerId
 * @param msg
 */
export declare const debugInfoRequestHandler: (params: {
    node: Libp2p;
    getPeerInfo: () => Promise<DebugPeerInfo>;
    peerId: PeerId;
    msg: any;
}) => Promise<void>;
/**
 * Method to get self node info
 * @param node
 * @returns
 */
export declare const getSelfInfo: (node: Libp2p) => SelfInfo;
/**
 * Method to get connections info
 * @param node
 * @param peerHeartbeatChecker
 * @returns
 */
export declare const getConnectionsInfo: (node: Libp2p, peerHeartbeatChecker: PeerHearbeatChecker) => ConnectionInfo[];
export declare const wsPeerFilter: (multiaddrs: Multiaddr[]) => Multiaddr[];
export declare const initPubsub: (pubsubType?: PubsubType, directPeers?: string[]) => any;
export {};
