//
// Copyright 2023 Vulcanize, Inc.
//
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
import * as mafmt from '@multiformats/mafmt';
import { uniqueNamesGenerator, adjectives, colors, names } from 'unique-names-generator';
import { fromString as uint8ArrayFromString } from 'uint8arrays/from-string';
import debug from 'debug';
import assert from 'assert';
import { peerIdFromString } from '@libp2p/peer-id';
import { floodsub } from '@libp2p/floodsub';
import { gossipsub } from '@chainsafe/libp2p-gossipsub';
import { ConnectionType } from '../types/debug-info.js';
import { DEBUG_INFO_TOPIC, DEFAULT_PUBSUB_TYPE, P2P_WEBRTC_STAR_ID, PUBSUB_SIGNATURE_POLICY } from '../constants.js';
var log = debug('laconic:utils');
// p2p multi-address codes
export var CODE_P2P = 421;
export var CODE_CIRCUIT = 290;
var ERR_INVALID_PUBSUB_TYPE = 'Invalid pubsub type';
var DEFAULT_DIAL_RETRY_OPTIONS = {
    redialInterval: 5000,
    maxRetry: 5
};
/**
 * Method to dial remote peer multiaddr with retry on failure
 * Number of retries can be configured using options.maxRetry
 * @param node
 * @param multiaddr
 * @param options
 */
export var dialWithRetry = function (node, multiaddr, options) { return __awaiter(void 0, void 0, void 0, function () {
    var _a, redialInterval, maxRetry, i, peerId, connection, err_1;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _a = __assign(__assign({}, DEFAULT_DIAL_RETRY_OPTIONS), options), redialInterval = _a.redialInterval, maxRetry = _a.maxRetry;
                i = 0;
                _b.label = 1;
            case 1:
                if (!(i < maxRetry)) return [3 /*break*/, 7];
                peerId = multiaddr.getPeerId();
                assert(peerId);
                _b.label = 2;
            case 2:
                _b.trys.push([2, 4, , 6]);
                log("Dialling node ".concat(peerId, " (").concat(getPseudonymForPeerId(peerId.toString()), ") using multiaddr ").concat(multiaddr.toString()));
                return [4 /*yield*/, node.dial(multiaddr)];
            case 3:
                connection = _b.sent();
                return [2 /*return*/, connection];
            case 4:
                err_1 = _b.sent();
                log("Could not dial node ".concat(multiaddr.toString(), " (").concat(getPseudonymForPeerId(peerId.toString()), ")"), err_1);
                log("Retrying after ".concat(redialInterval, "ms"));
                // TODO: Use wait method from util package.
                // Issue using util package in react app.
                return [4 /*yield*/, new Promise(function (resolve) { return setTimeout(resolve, redialInterval); })];
            case 5:
                // TODO: Use wait method from util package.
                // Issue using util package in react app.
                _b.sent();
                return [3 /*break*/, 6];
            case 6:
                i++;
                return [3 /*break*/, 1];
            case 7: throw new Error("Stopping dial retry after ".concat(maxRetry, " attempts for multiaddr ").concat(multiaddr.toString()));
        }
    });
}); };
/**
 * Get a deterministic pseudonym of form [adjective-color-name] for a given libp2p peer id
 * Eg. 12D3KooWJLXEX2GfHPSZR3z9QKNSN8EY6pXo7FZ9XtFhiKLJATtC -> jolly-green-diann
 * @param peerId
 */
export var getPseudonymForPeerId = function (peerId) {
    return uniqueNamesGenerator({
        seed: peerId,
        dictionaries: [adjectives, colors, names],
        length: 3,
        style: 'lowerCase',
        separator: '-'
    });
};
/**
 * Method to check if the given multiaddr is part of a blacklist
 * @param blacklist
 * @param multiaddr
 */
export var isMultiaddrBlacklisted = function (blacklist, multiaddr) {
    return blacklist.some(function (maString) {
        return multiaddr.toString().startsWith(maString);
    });
};
/**
 * Handler for pubsub debug info request
 * @param peerId
 * @param msg
 */
export var debugInfoRequestHandler = function (params) { return __awaiter(void 0, void 0, void 0, function () {
    var node, peerId, msg, getPeerInfo, debugMsg, msgType, peerInfo, response;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                node = params.node, peerId = params.peerId, msg = params.msg, getPeerInfo = params.getPeerInfo;
                debugMsg = msg;
                msgType = debugMsg.type;
                if (!(msgType === 'Request')) return [3 /*break*/, 3];
                log('got a debug info request from', peerId.toString(), "(".concat(getPseudonymForPeerId(peerId.toString()), ")"));
                return [4 /*yield*/, getPeerInfo()];
            case 1:
                peerInfo = _a.sent();
                response = {
                    type: 'Response',
                    dst: peerId.toString(),
                    peerInfo: peerInfo
                };
                return [4 /*yield*/, node.pubsub.publish(DEBUG_INFO_TOPIC, uint8ArrayFromString(JSON.stringify(response)))];
            case 2:
                _a.sent();
                _a.label = 3;
            case 3: return [2 /*return*/];
        }
    });
}); };
/**
 * Method to get self node info
 * @param node
 * @returns
 */
export var getSelfInfo = function (node) {
    return {
        peerId: node.peerId.toString(),
        multiaddrs: node.getMultiaddrs().map(function (multiaddr) { return multiaddr.toString(); })
    };
};
/**
 * Method to get connections info
 * @param node
 * @param peerHeartbeatChecker
 * @returns
 */
export var getConnectionsInfo = function (node, peerHeartbeatChecker) {
    return node.getConnections().map(function (connection) {
        return {
            id: connection.id,
            peerId: connection.remotePeer.toString(),
            multiaddr: connection.remoteAddr.toString(),
            direction: connection.stat.direction,
            status: connection.stat.status,
            type: connection.remoteAddr.toString().includes('p2p-circuit/p2p') ? ConnectionType.Relayed : ConnectionType.Direct,
            latency: peerHeartbeatChecker.getLatencyData(connection.remotePeer)
        };
    });
};
export var wsPeerFilter = function (multiaddrs) {
    return multiaddrs.filter(function (ma) {
        if (ma.protoCodes().includes(CODE_CIRCUIT)) {
            return false;
        }
        if (ma.protoNames().includes(P2P_WEBRTC_STAR_ID)) {
            return false;
        }
        var testMa = ma.decapsulateCode(CODE_P2P);
        return mafmt.WebSockets.matches(testMa) ||
            mafmt.WebSocketsSecure.matches(testMa);
    });
};
export var initPubsub = function (pubsubType, directPeers) {
    var pubsub;
    switch (pubsubType || DEFAULT_PUBSUB_TYPE) {
        case 'floodsub':
            pubsub = floodsub({ globalSignaturePolicy: PUBSUB_SIGNATURE_POLICY });
            break;
        case 'gossipsub':
            pubsub = gossipsub({
                globalSignaturePolicy: PUBSUB_SIGNATURE_POLICY,
                allowPublishToZeroPeers: true,
                directPeers: (directPeers === null || directPeers === void 0 ? void 0 : directPeers.map(function (directPeer) {
                    return { id: peerIdFromString(directPeer), addrs: [] };
                })) || []
            });
            break;
        default:
            throw new Error("".concat(ERR_INVALID_PUBSUB_TYPE, ": ").concat(pubsubType));
    }
    return pubsub;
};
//# sourceMappingURL=index.js.map